/*
	Copyright (C) 2007 Belhorma Bendebiche
	
	This file is part of FreeDL.

	FreeDL is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeDL is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeDL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "socket.h"

Socket::Socket()
{

}

Socket::Socket(string hostName, short port)
{
	SetRemote(hostName, port);
}

Socket::~Socket()
{
	Disconnect();
}

void Socket::SetRemote(string hostName, short port)
{
	he = gethostbyname(hostName.c_str());

	if (he == 0)
		throw DNSErrorException();
		
	host = he->h_name;	
	remoteAddress.sin_family = AF_INET;
	remoteAddress.sin_port = htons(port);
	remoteAddress.sin_addr = *((struct in_addr *)he->h_addr);
	memset(remoteAddress.sin_zero, '\0', sizeof remoteAddress.sin_zero);
	

}

void Socket::Connect()
{
	if (connect(sockfd, (struct sockaddr *)&remoteAddress, sizeof remoteAddress) == -1)
	{
		switch (errno)
		{
			case EWOULDBLOCK:
				throw WouldBlockException();
			
			case EINPROGRESS:
				throw InProgressException();
		}
		throw ConnectErrorException("Errno is: " + errno);
	}
}

void Socket::Disconnect()
{
	close(sockfd);
}

int Socket::Send(const void *data, int size)
{
	send(sockfd, data, size, 0);
}

int Socket::Receive(void *data, int size)
{
	recv(sockfd, data, size, 0);
}
